package com.hncy.demo.consumer.controller;

import com.hncy.auth.inject.spring.boot.starter.auth.AuthUser;
import com.hncy.auth.inject.spring.boot.starter.holder.AuthUserHolder;
import com.hncy.common.dto.ResultBean;
import com.hncy.demo.dto.DemoDTO;
import com.hncy.demo.feign.DemoFeignClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: ying.dong
 * @Date: 2021/4/14 18:02
 */
@Slf4j
@RequestMapping("consume")
@RestController
public class DemoConsumerController {

    @Autowired
    private AuthUserHolder authUserHolder;

    @Autowired
    private DemoFeignClient demoFeignClient;

    @GetMapping("{id}")
    public ResultBean<DemoDTO> consume(@PathVariable("id")Integer id) {
        return demoFeignClient.getDemo(id);
    }

    @PutMapping("{id}")
    public ResultBean<DemoDTO> modify(@PathVariable("id")Integer id, @RequestParam("desc")String desc) {
        return demoFeignClient.modify(id, desc);
    }

    @PostMapping
    public ResultBean<Void> create(@RequestParam("desc")String desc) {
        //demo for hard code of exception
        return demoFeignClient.create(desc);
    }

    @DeleteMapping("{id}")
    public ResultBean<Void> delete(@PathVariable("id")Integer id) {
        //demo for variable message of exception
        try {
            return demoFeignClient.delete(id);
        } catch (Exception e) {
            log.error("there's a problem while delete some items, which id {}", id, e);
            return ResultBean.fail(e);
        }
    }

    @GetMapping
    public ResultBean<AuthUser> me(AuthUser authUser) {
        log.info("auth user from holder: {}", authUserHolder.getAuthUser());
        return ResultBean.success(authUser);
    }
}
