package com.hncy.service.demo.controller;

import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.common.utils.IOUtils;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.demo.dto.DemoDTO;
import com.hncy.file.manage.spring.boot.starter.oss.manager.OSSManager;
import com.hncy.file.manage.spring.boot.starter.oss.resource.OSSResource;
import com.hncy.service.demo.constant.ErrorCodeConstants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * @Author: ying.dong
 * @Date: 2021/4/14 16:11
 */
@RequestMapping("demo")
@RestController
public class DemoController {

    @Autowired
    private OSSManager ossManager;

    @GetMapping("{id}")
    public ResultBean<DemoDTO> get(@PathVariable("id") Integer id) {
        return ResultBean.success(new DemoDTO("just a demo.", id));
    }

    @PutMapping("{id}")
    public ResultBean<DemoDTO> modify(@PathVariable("id")Integer id, @RequestParam("desc")String desc) {
        return ResultBean.success(new DemoDTO(desc, id));
    }

    @PostMapping
    public ResultBean<Void> create(@RequestParam("desc")String desc) {
//        demo for hard code of exception
//        return ResultBean.success("xxx");
        return ResultBean.fail(new BusinessException(600, "该ID已存在"));
    }

    @DeleteMapping("{id}")
    public ResultBean<Void> delete(@PathVariable("id")Integer id) {
        //demo for variable message of exception
        return ResultBean.fail(ErrorCodeConstants.DELETE_ERROR);
    }

    @GetMapping("resource")
    public ResultBean<OSSResource> testResource() {
        OSSResource ossResource = new OSSResource("oss.png", "images");
        ossResource = ossManager.get("test", ossResource);
        return ResultBean.success(ossResource);
    }

    @PutMapping("resource")
    public ResultBean<OSSResource> putResource() throws IOException {
        File file = new File("/Users/dy/oss.png");
        byte[] bytes = IOUtils.readStreamAsByteArray(new FileInputStream(file));
        byte[] md5Bytes = BinaryUtil.calculateMd5(bytes);
        String md5 = BinaryUtil.toBase64String(md5Bytes);
        OSSResource ossResource = new OSSResource("oss.png", "images");
        ossResource.setMD5(md5);
        OSSResource resource = ossManager.save("test", ossResource, true);
        return ResultBean.success(resource);
    }

    @DeleteMapping("resource")
    public ResultBean<Void> deleteResource() {
        OSSResource ossResource = new OSSResource("oss.png", "images");
        ossManager.del("test", ossResource);
        return ResultBean.success();
    }

}
