package com.hncy.service.wechat.applet.biz.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.hncy.common.dto.ResultBean;
import com.hncy.common.exception.custom.BusinessException;
import com.hncy.common.exception.custom.UnAuthorizedException;
import com.hncy.jackson.spring.boot.starter.core.JSONObject;
import com.hncy.service.wechat.applet.api.dto.output.Code2SessionOutput;
import com.hncy.service.wechat.applet.api.feign.WxAppletFeignClient;
import com.hncy.service.wechat.applet.api.vo.WxUserInfo;
import com.hncy.service.wechat.applet.api.vo.WxUserVo;
import com.hncy.service.wechat.applet.biz.entity.WxUser;
import com.hncy.service.wechat.applet.biz.mapper.WxUserMapper;
import com.hncy.service.wechat.applet.biz.service.ComponentsRemoteService;
import com.hncy.service.wechat.applet.biz.service.WxUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 微信小程序用户表 服务实现类
 * </p>
 *
 * @author cs
 * @since 2021-10-27
 */
@Service
@Slf4j
public class WxUserServiceImpl extends ServiceImpl<WxUserMapper, WxUser> implements WxUserService {

    @Autowired
    private WxUserMapper wxUserMapper;
    @Autowired
    private ComponentsRemoteService componentsRemoteService;
    @Autowired
    private WxAppletFeignClient wxAppletFeignClient;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultBean saveWxUserInfo(String openId, String avatar, String nickname, String sexCode, String sexName, String tel) {

        //判断该手机号是否被其他用户注册了
        WxUser wxUserTel = wxUserMapper.selectByTel(tel);
        if (ObjectUtil.isNotNull(wxUserTel)) {
            throw new BusinessException("该手机号已经被其他用户注册,请重新更换手机号");
        }
        // 通过openId获取用户信息;
        WxUser wxUser = wxUserMapper.getWxUserInfoByopenId(openId);
        if(ObjectUtil.isNull(wxUser)) {
            log.info("用户不存在,注册");
            WxUser newWxUser = new WxUser();
            newWxUser.setOpenId(openId);
            newWxUser.setDeleted(0);
            newWxUser.setAvatar(avatar);
            newWxUser.setGmtCreate(DateTime.now());
            newWxUser.setLoginDate(LocalDateTime.now());
            newWxUser.setSexCode(sexCode);
            newWxUser.setSexName(sexName);
            newWxUser.setNickname(nickname);
            newWxUser.setTel(tel);
            newWxUser.setCreateBy(openId);
            //录入数据库
            wxUserMapper.insert(newWxUser);
            return ResultBean.success("用户注册成功");
        }else {
            log.info("用户已存在，更新信息");
            WxUser newWxUser = new WxUser();
            newWxUser.setOpenId(openId);
            newWxUser.setDeleted(0);
            newWxUser.setAvatar(avatar);
            newWxUser.setLoginDate(LocalDateTime.now());
            newWxUser.setSexCode(sexCode);
            newWxUser.setSexName(sexName);
            newWxUser.setNickname(nickname);
            newWxUser.setTel(tel);
            newWxUser.setUpdateBy(openId);
            newWxUser.setGmtModified(DateTime.now());
            wxUserMapper.updateById(newWxUser);
            return ResultBean.success("用户更新成功");
        }
    }

    @Override
    public ResultBean<WxUserVo> booleanWxUserByOpenId(String openId) {
        WxUserVo wxUserVo = new WxUserVo();
        WxUser wxUser = wxUserMapper.selectOpenId(openId);
        if(null == wxUser) {
            return ResultBean.success(null);
        }
        BeanUtils.copyProperties(wxUser, wxUserVo);
        return ResultBean.success(wxUserVo);
    }

    @Override
    public ResultBean<WxUserInfo> getOpenId(String wxUserId) {
        WxUserInfo wxUserInfo = new WxUserInfo();
        String openId = wxUserMapper.getOpenId(wxUserId);
        if(!StringUtils.isEmpty(openId)) {
            wxUserInfo.setOpenId(openId);
        }
        return ResultBean.success(wxUserInfo);
    }

    @Override
    public ResultBean<String> getNickNameById(String id) {

        return null;
    }

    @Override
    public ResultBean appletLogin(String code) {
        Code2SessionOutput data = wxAppletFeignClient.login(code).getData();
        if (StrUtil.isBlank(data.getOpenid())) {
            throw new BusinessException("登录异常,小程序openId为空");
        }
        if (StrUtil.isBlank(data.getSessionKey())) {
            throw new BusinessException("登录异常,sessionkey值为空");
        }
        //token生效
        wxAppletFeignClient.getAccessToken();

        WxUserVo res = wxAppletFeignClient.booleanWxUserByOpenId(data.getOpenid()).getData();
        if (res == null || StrUtil.isBlank(res.getOpenId())) {
            ResultBean<WxUserVo> saveResult = wxAppletFeignClient.saveWxUser(data.getOpenid(),data.getUnionid());
            if(!saveResult.isSuccess()) {
                throw new BusinessException("登录异常");
            }
        }
        return ResultBean.success();
    }

    @Override
    public ResultBean get() {
        List<Map<String, Object>> maps = componentsRemoteService.loadAllComponents();
        String s = JSONObject.getInstance().toJSONString(maps);
        log.info(s);
        return ResultBean.success(maps);
    }

    @Override
    public WxUser getWxUserInfoByopenId(String openId) {
        WxUser wxUser = wxUserMapper.getWxUserInfoByopenId(openId);
        if(null == wxUser) {
            throw new UnAuthorizedException("cannot find wxUser with openId: " + openId);
        }
        return wxUser;
    }
}
